//! This module contains everything related to scenes.

use std::slice::{Iter, IterMut};
use std::vec::IntoIter;
use std::sync::Arc;
use std::cell::RefCell;
use model::Model;

/// Represents a 3D scene with models.
pub struct Scene {
    models: Vec<Arc<RefCell<Model>>>,
}

impl Scene {
    /// Creates a new empty scene.
    pub fn new() -> Scene {
        Scene {
            models: vec![],
        }
    }

    /// Adds a model to the scene.
    pub fn push(&mut self, model: Arc<RefCell<Model>>) {
        self.models.push(model);
    }

    /// Converts the model to a Arc<RefCell<Model>> and adds it to the scene.
    pub fn emplace(&mut self, model: Model) {
        self.models.push(Arc::new(RefCell::new(model)));
    }

    /// Returns an iterator to the models of the scene.
    pub fn iter(&self) -> Iter<Arc<RefCell<Model>>> {
        self.models.iter()
    }

    /// Returns a mutable iterator to the model of the scene.
    pub fn iter_mut(&mut self) -> IterMut<Arc<RefCell<Model>>> {
        self.models.iter_mut()
    }
}

impl IntoIterator for Scene {
    type IntoIter = IntoIter<Arc<RefCell<Model>>>;
    type Item = Arc<RefCell<Model>>;

    fn into_iter(self) -> Self::IntoIter {
        self.models.into_iter()
    }
}

impl From<Vec<Model>> for Scene {
    fn from(input: Vec<Model>) -> Scene {
        Scene {
            models: input.into_iter().map(|x| Arc::new(RefCell::new(x))).collect(),
        }
    }
}
